////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Copyright (c) Qualcomm Technologies, Inc. and/or its subsidiaries.
// All Rights Reserved.
// Confidential and Proprietary - Qualcomm Technologies, Inc.
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// THIS IS AUTO-GENERATED CODE.  PLEASE DO NOT EDIT (File bug reports against tools).
///
/// Auto-generated by: Parameter Parser V5.5.1 (2411131018)
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// @file  tmc_1_0_0.cpp
/// @brief Auto-generated Chromatix parameter file
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#include "tmc_1_0_0.h"
#include "parametersetmanager.h"

TMC_1_0_0_NAMESPACE_BEGIN

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// SetValue/GetValue Functions
/// Gets or sets the requested value
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool tmc10_rgn_dataType::GetValue(
    const CHAR* const __path,
    CHAR* const       __value,
    UINT32 const      __size)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "gtm_percentage") == 0)
    {
        ParameterUtils::DoubleToString((double)gtm_percentage, __value, __size);
        __result = true;
    }
    else if (PARAMETER_STRCMP(__name, "ltm_percentage") == 0)
    {
        ParameterUtils::DoubleToString((double)ltm_percentage, __value, __size);
        __result = true;
    }
    else if (PARAMETER_STRCMP(__name, "la_percentage") == 0)
    {
        ParameterUtils::DoubleToString((double)la_percentage, __value, __size);
        __result = true;
    }
    else if (PARAMETER_STRCMP(__name, "gamma_percentage") == 0)
    {
        ParameterUtils::DoubleToString((double)gamma_percentage, __value, __size);
        __result = true;
    }
    else if (PARAMETER_STRCMP(__name, "knee_out_mid") == 0)
    {
        ParameterUtils::DoubleToString((double)knee_out_mid, __value, __size);
        __result = true;
    }
    else if (PARAMETER_STRCMP(__name, "dark_boost_ratio") == 0)
    {
        ParameterUtils::DoubleToString((double)dark_boost_ratio, __value, __size);
        __result = true;
    }
    else if (PARAMETER_STRCMP(__name, "core_rsv_para1") == 0)
    {
        ParameterUtils::DoubleToString((double)core_rsv_para1, __value, __size);
        __result = true;
    }
    else if (PARAMETER_STRCMP(__name, "core_rsv_para2") == 0)
    {
        ParameterUtils::DoubleToString((double)core_rsv_para2, __value, __size);
        __result = true;
    }
    else if (PARAMETER_STRCMP(__name, "core_rsv_para3") == 0)
    {
        ParameterUtils::DoubleToString((double)core_rsv_para3, __value, __size);
        __result = true;
    }
    else if (PARAMETER_STRCMP(__name, "core_rsv_para4") == 0)
    {
        ParameterUtils::DoubleToString((double)core_rsv_para4, __value, __size);
        __result = true;
    }
    else if (PARAMETER_STRCMP(__name, "core_rsv_para5") == 0)
    {
        ParameterUtils::DoubleToString((double)core_rsv_para5, __value, __size);
        __result = true;
    }

    return __result;
}

bool tmc10_rgn_dataType::SetValue(
    const CHAR* const __path,
    const CHAR* const __value)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "gtm_percentage") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            gtm_percentage = (float)dblValue; 
        }
    }
    else if (PARAMETER_STRCMP(__name, "ltm_percentage") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            ltm_percentage = (float)dblValue; 
        }
    }
    else if (PARAMETER_STRCMP(__name, "la_percentage") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            la_percentage = (float)dblValue; 
        }
    }
    else if (PARAMETER_STRCMP(__name, "gamma_percentage") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            gamma_percentage = (float)dblValue; 
        }
    }
    else if (PARAMETER_STRCMP(__name, "knee_out_mid") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            knee_out_mid = (float)dblValue; 
        }
    }
    else if (PARAMETER_STRCMP(__name, "dark_boost_ratio") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            dark_boost_ratio = (float)dblValue; 
        }
    }
    else if (PARAMETER_STRCMP(__name, "core_rsv_para1") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            core_rsv_para1 = (float)dblValue; 
        }
    }
    else if (PARAMETER_STRCMP(__name, "core_rsv_para2") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            core_rsv_para2 = (float)dblValue; 
        }
    }
    else if (PARAMETER_STRCMP(__name, "core_rsv_para3") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            core_rsv_para3 = (float)dblValue; 
        }
    }
    else if (PARAMETER_STRCMP(__name, "core_rsv_para4") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            core_rsv_para4 = (float)dblValue; 
        }
    }
    else if (PARAMETER_STRCMP(__name, "core_rsv_para5") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            core_rsv_para5 = (float)dblValue; 
        }
    }

    return __result;
}

bool tmc10_rgn_dataType::GetInfo(
    const CHAR* const           __path,
    PList<ParameterParserInfo>& __info)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    // gtm_percentage
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("gtm_percentage", ParameterParserDataType::Float, (double)gtm_percentage));
    }
    else if (PARAMETER_STRCMP(__name, "gtm_percentage") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Float, (double)gtm_percentage));
    }

    // ltm_percentage
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("ltm_percentage", ParameterParserDataType::Float, (double)ltm_percentage));
    }
    else if (PARAMETER_STRCMP(__name, "ltm_percentage") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Float, (double)ltm_percentage));
    }

    // la_percentage
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("la_percentage", ParameterParserDataType::Float, (double)la_percentage));
    }
    else if (PARAMETER_STRCMP(__name, "la_percentage") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Float, (double)la_percentage));
    }

    // gamma_percentage
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("gamma_percentage", ParameterParserDataType::Float, (double)gamma_percentage));
    }
    else if (PARAMETER_STRCMP(__name, "gamma_percentage") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Float, (double)gamma_percentage));
    }

    // knee_out_mid
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("knee_out_mid", ParameterParserDataType::Float, (double)knee_out_mid));
    }
    else if (PARAMETER_STRCMP(__name, "knee_out_mid") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Float, (double)knee_out_mid));
    }

    // dark_boost_ratio
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("dark_boost_ratio", ParameterParserDataType::Float, (double)dark_boost_ratio));
    }
    else if (PARAMETER_STRCMP(__name, "dark_boost_ratio") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Float, (double)dark_boost_ratio));
    }

    // core_rsv_para1
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("core_rsv_para1", ParameterParserDataType::Float, (double)core_rsv_para1));
    }
    else if (PARAMETER_STRCMP(__name, "core_rsv_para1") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Float, (double)core_rsv_para1));
    }

    // core_rsv_para2
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("core_rsv_para2", ParameterParserDataType::Float, (double)core_rsv_para2));
    }
    else if (PARAMETER_STRCMP(__name, "core_rsv_para2") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Float, (double)core_rsv_para2));
    }

    // core_rsv_para3
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("core_rsv_para3", ParameterParserDataType::Float, (double)core_rsv_para3));
    }
    else if (PARAMETER_STRCMP(__name, "core_rsv_para3") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Float, (double)core_rsv_para3));
    }

    // core_rsv_para4
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("core_rsv_para4", ParameterParserDataType::Float, (double)core_rsv_para4));
    }
    else if (PARAMETER_STRCMP(__name, "core_rsv_para4") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Float, (double)core_rsv_para4));
    }

    // core_rsv_para5
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("core_rsv_para5", ParameterParserDataType::Float, (double)core_rsv_para5));
    }
    else if (PARAMETER_STRCMP(__name, "core_rsv_para5") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Float, (double)core_rsv_para5));
    }

    return __result;
}

bool mod_tmc10_aec_dataType::GetValue(
    const CHAR* const __path,
    CHAR* const       __value,
    UINT32 const      __size)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    const UINT32 __pos = ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "aec_trigger") == 0)
    {
        __result = aec_trigger.GetValue(&__path[__pos], __value, __size);
    }
    else if (PARAMETER_STRCMP(__name, "tmc10_rgn_data") == 0)
    {
        __result = tmc10_rgn_data.GetValue(&__path[__pos], __value, __size);
    }

    return __result;
}

bool mod_tmc10_aec_dataType::SetValue(
    const CHAR* const __path,
    const CHAR* const __value)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    const UINT32 __pos = ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "aec_trigger") == 0)
    {
        __result = aec_trigger.SetValue(&__path[__pos], __value);
    }
    else if (PARAMETER_STRCMP(__name, "tmc10_rgn_data") == 0)
    {
        __result = tmc10_rgn_data.SetValue(&__path[__pos], __value);
    }

    return __result;
}

bool mod_tmc10_aec_dataType::GetInfo(
    const CHAR* const           __path,
    PList<ParameterParserInfo>& __info)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    const UINT32 __pos = ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    // aec_trigger
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("aec_trigger", ParameterParserDataType::Structure));
    }
    else if (PARAMETER_STRCMP(__name, "aec_trigger") == 0)
    {
        __result = aec_trigger.GetInfo(&__path[__pos], __info);
    }

    // tmc10_rgn_data
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("tmc10_rgn_data", ParameterParserDataType::Structure));
    }
    else if (PARAMETER_STRCMP(__name, "tmc10_rgn_data") == 0)
    {
        __result = tmc10_rgn_data.GetInfo(&__path[__pos], __info);
    }

    return __result;
}

bool chromatix_tmc10_coreType::GetValue(
    const CHAR* const __path,
    CHAR* const       __value,
    UINT32 const      __size)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    const UINT32 __pos = ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "mod_tmc10_aec_data") == 0)
    {
        if (__index < mod_tmc10_aec_dataCount)
        {
            __result = mod_tmc10_aec_data[__index].GetValue(&__path[__pos], __value, __size);
        }
    }

    return __result;
}

bool chromatix_tmc10_coreType::SetValue(
    const CHAR* const __path,
    const CHAR* const __value)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    const UINT32 __pos = ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "mod_tmc10_aec_data") == 0)
    {
        if (__index < mod_tmc10_aec_dataCount)
        {
            __result = mod_tmc10_aec_data[__index].SetValue(&__path[__pos], __value);
        }
    }

    return __result;
}

bool chromatix_tmc10_coreType::GetInfo(
    const CHAR* const           __path,
    PList<ParameterParserInfo>& __info)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    const UINT32 __pos = ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    // mod_tmc10_aec_data
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("mod_tmc10_aec_data", ParameterParserDataType::Structure, true, false, mod_tmc10_aec_dataCount));
    }
    else if (PARAMETER_STRCMP(__name, "mod_tmc10_aec_data") == 0)
    {
        if (__index < mod_tmc10_aec_dataCount)
        {
            __result = mod_tmc10_aec_data[__index].GetInfo(&__path[__pos], __info);
        }
        else
        {
            __result = true;
            __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Structure, true, false, mod_tmc10_aec_dataCount));
        }
    }

    return __result;
}

bool chromatix_tmc10_reserveType::GetValue(
    const CHAR* const __path,
    CHAR* const       __value,
    UINT32 const      __size)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "use_gtm") == 0)
    {
        ParameterUtils::DoubleToString((double)use_gtm, __value, __size, 0);
        __result = true;
    }
    else if (PARAMETER_STRCMP(__name, "use_ltm") == 0)
    {
        ParameterUtils::DoubleToString((double)use_ltm, __value, __size, 0);
        __result = true;
    }
    else if (PARAMETER_STRCMP(__name, "use_la") == 0)
    {
        ParameterUtils::DoubleToString((double)use_la, __value, __size, 0);
        __result = true;
    }
    else if (PARAMETER_STRCMP(__name, "use_gamma") == 0)
    {
        ParameterUtils::DoubleToString((double)use_gamma, __value, __size, 0);
        __result = true;
    }
    else if (PARAMETER_STRCMP(__name, "reserve_rsv_para1") == 0)
    {
        ParameterUtils::DoubleToString((double)reserve_rsv_para1, __value, __size);
        __result = true;
    }
    else if (PARAMETER_STRCMP(__name, "reserve_rsv_para2") == 0)
    {
        ParameterUtils::DoubleToString((double)reserve_rsv_para2, __value, __size);
        __result = true;
    }
    else if (PARAMETER_STRCMP(__name, "reserve_rsv_para3") == 0)
    {
        ParameterUtils::DoubleToString((double)reserve_rsv_para3, __value, __size);
        __result = true;
    }
    else if (PARAMETER_STRCMP(__name, "reserve_rsv_para4") == 0)
    {
        ParameterUtils::DoubleToString((double)reserve_rsv_para4, __value, __size);
        __result = true;
    }
    else if (PARAMETER_STRCMP(__name, "reserve_rsv_para5") == 0)
    {
        ParameterUtils::DoubleToString((double)reserve_rsv_para5, __value, __size);
        __result = true;
    }

    return __result;
}

bool chromatix_tmc10_reserveType::SetValue(
    const CHAR* const __path,
    const CHAR* const __value)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "use_gtm") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            use_gtm = (UINT32)dblValue; 
        }
    }
    else if (PARAMETER_STRCMP(__name, "use_ltm") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            use_ltm = (UINT32)dblValue; 
        }
    }
    else if (PARAMETER_STRCMP(__name, "use_la") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            use_la = (UINT32)dblValue; 
        }
    }
    else if (PARAMETER_STRCMP(__name, "use_gamma") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            use_gamma = (UINT32)dblValue; 
        }
    }
    else if (PARAMETER_STRCMP(__name, "reserve_rsv_para1") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            reserve_rsv_para1 = (float)dblValue; 
        }
    }
    else if (PARAMETER_STRCMP(__name, "reserve_rsv_para2") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            reserve_rsv_para2 = (float)dblValue; 
        }
    }
    else if (PARAMETER_STRCMP(__name, "reserve_rsv_para3") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            reserve_rsv_para3 = (float)dblValue; 
        }
    }
    else if (PARAMETER_STRCMP(__name, "reserve_rsv_para4") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            reserve_rsv_para4 = (float)dblValue; 
        }
    }
    else if (PARAMETER_STRCMP(__name, "reserve_rsv_para5") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            reserve_rsv_para5 = (float)dblValue; 
        }
    }

    return __result;
}

bool chromatix_tmc10_reserveType::GetInfo(
    const CHAR* const           __path,
    PList<ParameterParserInfo>& __info)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    // use_gtm
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("use_gtm", ParameterParserDataType::UInt32, (double)use_gtm, 0));
    }
    else if (PARAMETER_STRCMP(__name, "use_gtm") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::UInt32, (double)use_gtm, 0));
    }

    // use_ltm
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("use_ltm", ParameterParserDataType::UInt32, (double)use_ltm, 0));
    }
    else if (PARAMETER_STRCMP(__name, "use_ltm") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::UInt32, (double)use_ltm, 0));
    }

    // use_la
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("use_la", ParameterParserDataType::UInt32, (double)use_la, 0));
    }
    else if (PARAMETER_STRCMP(__name, "use_la") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::UInt32, (double)use_la, 0));
    }

    // use_gamma
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("use_gamma", ParameterParserDataType::UInt32, (double)use_gamma, 0));
    }
    else if (PARAMETER_STRCMP(__name, "use_gamma") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::UInt32, (double)use_gamma, 0));
    }

    // reserve_rsv_para1
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("reserve_rsv_para1", ParameterParserDataType::Float, (double)reserve_rsv_para1));
    }
    else if (PARAMETER_STRCMP(__name, "reserve_rsv_para1") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Float, (double)reserve_rsv_para1));
    }

    // reserve_rsv_para2
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("reserve_rsv_para2", ParameterParserDataType::Float, (double)reserve_rsv_para2));
    }
    else if (PARAMETER_STRCMP(__name, "reserve_rsv_para2") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Float, (double)reserve_rsv_para2));
    }

    // reserve_rsv_para3
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("reserve_rsv_para3", ParameterParserDataType::Float, (double)reserve_rsv_para3));
    }
    else if (PARAMETER_STRCMP(__name, "reserve_rsv_para3") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Float, (double)reserve_rsv_para3));
    }

    // reserve_rsv_para4
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("reserve_rsv_para4", ParameterParserDataType::Float, (double)reserve_rsv_para4));
    }
    else if (PARAMETER_STRCMP(__name, "reserve_rsv_para4") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Float, (double)reserve_rsv_para4));
    }

    // reserve_rsv_para5
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("reserve_rsv_para5", ParameterParserDataType::Float, (double)reserve_rsv_para5));
    }
    else if (PARAMETER_STRCMP(__name, "reserve_rsv_para5") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Float, (double)reserve_rsv_para5));
    }

    return __result;
}

bool chromatix_tmc10Type::GetValue(
    const CHAR* const __path,
    CHAR* const       __value,
    UINT32 const      __size)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    const UINT32 __pos = ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "enable_section") == 0)
    {
        __result = enable_section.GetValue(&__path[__pos], __value, __size);
    }
    else if (PARAMETER_STRCMP(__name, "module_version") == 0)
    {
        __result = module_version.GetValue(&__path[__pos], __value, __size);
    }
    else if (PARAMETER_STRCMP(__name, "parameter_version") == 0)
    {
        __result = parameter_version.GetValue(&__path[__pos], __value, __size);
    }
    else if (PARAMETER_STRCMP(__name, "control_variables") == 0)
    {
        __result = control_variables.GetValue(&__path[__pos], __value, __size);
    }
    else if (PARAMETER_STRCMP(__name, "control_method") == 0)
    {
        __result = control_method.GetValue(&__path[__pos], __value, __size);
    }
    else if (PARAMETER_STRCMP(__name, "chromatix_tmc10_reserve") == 0)
    {
        __result = chromatix_tmc10_reserve.GetValue(&__path[__pos], __value, __size);
    }
    else if (PARAMETER_STRCMP(__name, "chromatix_tmc10_core") == 0)
    {
        __result = chromatix_tmc10_core.GetValue(&__path[__pos], __value, __size);
    }

    return __result;
}

bool chromatix_tmc10Type::SetValue(
    const CHAR* const __path,
    const CHAR* const __value)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    const UINT32 __pos = ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "enable_section") == 0)
    {
        __result = enable_section.SetValue(&__path[__pos], __value);
    }
    else if (PARAMETER_STRCMP(__name, "module_version") == 0)
    {
        __result = module_version.SetValue(&__path[__pos], __value);
    }
    else if (PARAMETER_STRCMP(__name, "parameter_version") == 0)
    {
        __result = parameter_version.SetValue(&__path[__pos], __value);
    }
    else if (PARAMETER_STRCMP(__name, "control_variables") == 0)
    {
        __result = control_variables.SetValue(&__path[__pos], __value);
    }
    else if (PARAMETER_STRCMP(__name, "control_method") == 0)
    {
        __result = control_method.SetValue(&__path[__pos], __value);
    }
    else if (PARAMETER_STRCMP(__name, "chromatix_tmc10_reserve") == 0)
    {
        __result = chromatix_tmc10_reserve.SetValue(&__path[__pos], __value);
    }
    else if (PARAMETER_STRCMP(__name, "chromatix_tmc10_core") == 0)
    {
        __result = chromatix_tmc10_core.SetValue(&__path[__pos], __value);
    }

    return __result;
}

bool chromatix_tmc10Type::GetInfo(
    const CHAR* const           __path,
    PList<ParameterParserInfo>& __info)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    const UINT32 __pos = ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    // enable_section
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("enable_section", ParameterParserDataType::Structure));
    }
    else if (PARAMETER_STRCMP(__name, "enable_section") == 0)
    {
        __result = enable_section.GetInfo(&__path[__pos], __info);
    }

    // module_version
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("module_version", ParameterParserDataType::Structure));
    }
    else if (PARAMETER_STRCMP(__name, "module_version") == 0)
    {
        __result = module_version.GetInfo(&__path[__pos], __info);
    }

    // parameter_version
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("parameter_version", ParameterParserDataType::Structure));
    }
    else if (PARAMETER_STRCMP(__name, "parameter_version") == 0)
    {
        __result = parameter_version.GetInfo(&__path[__pos], __info);
    }

    // control_variables
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("control_variables", ParameterParserDataType::Structure));
    }
    else if (PARAMETER_STRCMP(__name, "control_variables") == 0)
    {
        __result = control_variables.GetInfo(&__path[__pos], __info);
    }

    // control_method
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("control_method", ParameterParserDataType::Structure));
    }
    else if (PARAMETER_STRCMP(__name, "control_method") == 0)
    {
        __result = control_method.GetInfo(&__path[__pos], __info);
    }

    // chromatix_tmc10_reserve
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("chromatix_tmc10_reserve", ParameterParserDataType::Structure));
    }
    else if (PARAMETER_STRCMP(__name, "chromatix_tmc10_reserve") == 0)
    {
        __result = chromatix_tmc10_reserve.GetInfo(&__path[__pos], __info);
    }

    // chromatix_tmc10_core
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("chromatix_tmc10_core", ParameterParserDataType::Structure));
    }
    else if (PARAMETER_STRCMP(__name, "chromatix_tmc10_core") == 0)
    {
        __result = chromatix_tmc10_core.GetInfo(&__path[__pos], __info);
    }

    return __result;
}

bool chromatix_tmc10Type::enable_sectionStruct::GetValue(
    const CHAR* const __path,
    CHAR* const       __value,
    UINT32 const      __size)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "adrc_isp_enable") == 0)
    {
        ParameterUtils::DoubleToString((double)adrc_isp_enable, __value, __size, 0);
        __result = true;
    }

    return __result;
}

bool chromatix_tmc10Type::enable_sectionStruct::SetValue(
    const CHAR* const __path,
    const CHAR* const __value)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "adrc_isp_enable") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            adrc_isp_enable = (globalelements::enable_flag_type)dblValue; 
        }
    }

    return __result;
}

bool chromatix_tmc10Type::enable_sectionStruct::GetInfo(
    const CHAR* const           __path,
    PList<ParameterParserInfo>& __info)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    // adrc_isp_enable
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("adrc_isp_enable", ParameterParserDataType::Int32, (double)adrc_isp_enable, 0));
    }
    else if (PARAMETER_STRCMP(__name, "adrc_isp_enable") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Int32, (double)adrc_isp_enable, 0));
    }

    return __result;
}

bool chromatix_tmc10Type::module_versionStruct::GetValue(
    const CHAR* const __path,
    CHAR* const       __value,
    UINT32 const      __size)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "major_revision") == 0)
    {
        ParameterUtils::DoubleToString((double)major_revision, __value, __size, 0);
        __result = true;
    }
    else if (PARAMETER_STRCMP(__name, "minor_revision") == 0)
    {
        ParameterUtils::DoubleToString((double)minor_revision, __value, __size, 0);
        __result = true;
    }
    else if (PARAMETER_STRCMP(__name, "incr_revision") == 0)
    {
        ParameterUtils::DoubleToString((double)incr_revision, __value, __size, 0);
        __result = true;
    }

    return __result;
}

bool chromatix_tmc10Type::module_versionStruct::SetValue(
    const CHAR* const __path,
    const CHAR* const __value)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "major_revision") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            major_revision = (INT32)dblValue; 
        }
    }
    else if (PARAMETER_STRCMP(__name, "minor_revision") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            minor_revision = (INT32)dblValue; 
        }
    }
    else if (PARAMETER_STRCMP(__name, "incr_revision") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            incr_revision = (INT32)dblValue; 
        }
    }

    return __result;
}

bool chromatix_tmc10Type::module_versionStruct::GetInfo(
    const CHAR* const           __path,
    PList<ParameterParserInfo>& __info)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    // major_revision
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("major_revision", ParameterParserDataType::Int32, (double)major_revision, 0));
    }
    else if (PARAMETER_STRCMP(__name, "major_revision") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Int32, (double)major_revision, 0));
    }

    // minor_revision
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("minor_revision", ParameterParserDataType::Int32, (double)minor_revision, 0));
    }
    else if (PARAMETER_STRCMP(__name, "minor_revision") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Int32, (double)minor_revision, 0));
    }

    // incr_revision
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("incr_revision", ParameterParserDataType::Int32, (double)incr_revision, 0));
    }
    else if (PARAMETER_STRCMP(__name, "incr_revision") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Int32, (double)incr_revision, 0));
    }

    return __result;
}

bool chromatix_tmc10Type::parameter_versionStruct::GetValue(
    const CHAR* const __path,
    CHAR* const       __value,
    UINT32 const      __size)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "revision") == 0)
    {
        PARAMETER_STRCPY(__value, __size, revision);
        __result = true;
    }

    return __result;
}

bool chromatix_tmc10Type::parameter_versionStruct::SetValue(
    const CHAR* const __path,
    const CHAR* const /*__value*/)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "revision") == 0)
    {
        // Cannot modify strings, risk of memory leaks or corruption
    }

    return __result;
}

bool chromatix_tmc10Type::parameter_versionStruct::GetInfo(
    const CHAR* const           __path,
    PList<ParameterParserInfo>& __info)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    // revision
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("revision", ParameterParserDataType::String, revision));
    }
    else if (PARAMETER_STRCMP(__name, "revision") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::String, revision));
    }

    return __result;
}

bool chromatix_tmc10Type::control_variablesStruct::GetValue(
    const CHAR* const __path,
    CHAR* const       __value,
    UINT32 const      __size)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "control_var_type") == 0)
    {
        ParameterUtils::DoubleToString((double)(UINT32)control_var_type, __value, __size, 0);
        __result = true;
    }

    return __result;
}

bool chromatix_tmc10Type::control_variablesStruct::SetValue(
    const CHAR* const __path,
    const CHAR* const __value)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "control_var_type") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            control_var_type = (control_var_typeEnum)(UINT32)dblValue; 
        }
    }

    return __result;
}

bool chromatix_tmc10Type::control_variablesStruct::GetInfo(
    const CHAR* const           __path,
    PList<ParameterParserInfo>& __info)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    // control_var_type
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("control_var_type", ParameterParserDataType::Enum, (double)(UINT32)control_var_type, 0));
    }
    else if (PARAMETER_STRCMP(__name, "control_var_type") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Enum, (double)(UINT32)control_var_type, 0));
    }

    return __result;
}

bool chromatix_tmc10Type::control_methodStruct::GetValue(
    const CHAR* const __path,
    CHAR* const       __value,
    UINT32 const      __size)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "aec_exp_control") == 0)
    {
        ParameterUtils::DoubleToString((double)(UINT32)aec_exp_control, __value, __size, 0);
        __result = true;
    }

    return __result;
}

bool chromatix_tmc10Type::control_methodStruct::SetValue(
    const CHAR* const __path,
    const CHAR* const __value)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    if (PARAMETER_STRCMP(__name, "aec_exp_control") == 0)
    {
        const double dblValue = ParameterUtils::StringToDouble(__value, __result);
        if (__result)
        {
            aec_exp_control = (ispglobalelements::tuning_control_aec_type)(UINT32)dblValue; 
        }
    }

    return __result;
}

bool chromatix_tmc10Type::control_methodStruct::GetInfo(
    const CHAR* const           __path,
    PList<ParameterParserInfo>& __info)
{
    constexpr SIZE_T MAX_SIZE = 129;
    CHAR   __name[MAX_SIZE] = "";
    UINT32 __index          = MAX_UINT;
    bool   __result         = false;

    ParameterUtils::GetVariable(__path, __name, __index, MAX_SIZE);

    // aec_exp_control
    if (__name[0] == '\0')
    {
        __result = true;
        __info.Add(ParameterParserInfo("aec_exp_control", ParameterParserDataType::Enum, (double)(UINT32)aec_exp_control, 0));
    }
    else if (PARAMETER_STRCMP(__name, "aec_exp_control") == 0)
    {
        __result = true;
        __info.Add(ParameterParserInfo(nullptr, ParameterParserDataType::Enum, (double)(UINT32)aec_exp_control, 0));
    }

    return __result;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// @brief Read enum value
/// @param entry [in] Symbol table entry
/// @param enumValue [out] Enum value to read
/// @param alignment [in] Symbol table byte alignment
/// @return true if successful
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool chromatix_tmc10Type::control_variablesStruct::ReadEnum_control_var_typeEnum(
    ParameterFileSymbolTableEntry* const entry,
    chromatix_tmc10Type::control_variablesStruct::control_var_typeEnum& enumValue,
    BYTE const alignment)
{
    bool result {false};

    UINT32 uintValue {entry->ReadEnum(&result, alignment)};

    if (result)
    {
        switch (uintValue)
        {
        case static_cast<UINT32>(chromatix_tmc10Type::control_variablesStruct::control_var_typeEnum::control_lux_idx):
            enumValue = chromatix_tmc10Type::control_variablesStruct::control_var_typeEnum::control_lux_idx;
            result    = true;
            break;
        case static_cast<UINT32>(chromatix_tmc10Type::control_variablesStruct::control_var_typeEnum::control_gain):
            enumValue = chromatix_tmc10Type::control_variablesStruct::control_var_typeEnum::control_gain;
            result    = true;
            break;
        default:
            PARAMETER_LOG_ERROR(entry->SetManager, "Invalid value(% u) converting UINT32 to chromatix_tmc10Type::control_variablesStruct::control_var_typeEnum", uintValue);
            break;
        }
    }
    return result;
}


////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Default Constructor
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
chromatix_tmc10TypeClass::chromatix_tmc10TypeClass(
    const CHAR* const name)
    : ParameterModule(name, MAJOR, MINOR, PATCH, nullptr)
{
    PARAMETER_INITIALIZE(SymbolTableID);
    PARAMETER_INITIALIZE(enable_section);
    PARAMETER_INITIALIZE(module_version);
    PARAMETER_INITIALIZE(parameter_version);
    PARAMETER_INITIALIZE(control_variables);
    PARAMETER_INITIALIZE(control_method);
    PARAMETER_INITIALIZE(chromatix_tmc10_reserve);
    PARAMETER_INITIALIZE(chromatix_tmc10_core);
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Constructor
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
chromatix_tmc10TypeClass::chromatix_tmc10TypeClass(
    const CHAR* const name,
    ModuleData* const moduleData)
    : ParameterModule(name, MAJOR, MINOR, PATCH, moduleData)
{
    PARAMETER_INITIALIZE(SymbolTableID);
    PARAMETER_INITIALIZE(enable_section);
    PARAMETER_INITIALIZE(module_version);
    PARAMETER_INITIALIZE(parameter_version);
    PARAMETER_INITIALIZE(control_variables);
    PARAMETER_INITIALIZE(control_method);
    PARAMETER_INITIALIZE(chromatix_tmc10_reserve);
    PARAMETER_INITIALIZE(chromatix_tmc10_core);
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Destructor
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
chromatix_tmc10TypeClass::~chromatix_tmc10TypeClass()
{
    if (Loaded)
    {
        UnloadModule();
    }
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// GetName
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
const CHAR* chromatix_tmc10TypeClass::GetName(
    const CHAR* const type)
{
    const CHAR* name = type;
    CHAR temp[ParameterFileSymbolTableEntry::TYPE_LEN + 1];
    if (PARAMETER_STRCMP(type, ModuleData::GetType(temp, "tmc10_sw")) == 0)
    {
        name = "tmc10_sw";
    }
    return name;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Load tmc10_rgn_dataType
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool tmc10_rgn_dataType::Load(
    ParameterFileSymbolTableEntry* const entry,
    BYTE const alignment)
{
    bool result = true;

    PARAMETER_INITIALIZE(gtm_percentage);
    PARAMETER_INITIALIZE(ltm_percentage);
    PARAMETER_INITIALIZE(la_percentage);
    PARAMETER_INITIALIZE(gamma_percentage);
    PARAMETER_INITIALIZE(knee_out_mid);
    PARAMETER_INITIALIZE(dark_boost_ratio);
    PARAMETER_INITIALIZE(core_rsv_para1);
    PARAMETER_INITIALIZE(core_rsv_para2);
    PARAMETER_INITIALIZE(core_rsv_para3);
    PARAMETER_INITIALIZE(core_rsv_para4);
    PARAMETER_INITIALIZE(core_rsv_para5);

    SYMBOL_TABLE_ENTRY_READ(entry, result, gtm_percentage, alignment)
    SYMBOL_TABLE_ENTRY_READ(entry, result, ltm_percentage, alignment)
    SYMBOL_TABLE_ENTRY_READ(entry, result, la_percentage, alignment)
    SYMBOL_TABLE_ENTRY_READ(entry, result, gamma_percentage, alignment)
    SYMBOL_TABLE_ENTRY_READ(entry, result, knee_out_mid, alignment)
    SYMBOL_TABLE_ENTRY_READ(entry, result, dark_boost_ratio, alignment)
    SYMBOL_TABLE_ENTRY_READ(entry, result, core_rsv_para1, alignment)
    SYMBOL_TABLE_ENTRY_READ(entry, result, core_rsv_para2, alignment)
    SYMBOL_TABLE_ENTRY_READ(entry, result, core_rsv_para3, alignment)
    SYMBOL_TABLE_ENTRY_READ(entry, result, core_rsv_para4, alignment)
    SYMBOL_TABLE_ENTRY_READ(entry, result, core_rsv_para5, alignment)

    return result;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Load mod_tmc10_aec_dataType
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool mod_tmc10_aec_dataType::Load(
    ParameterFileSymbolTableEntry* const entry,
    BYTE const alignment)
{
    bool result = true;

    PARAMETER_INITIALIZE(aec_trigger);
    PARAMETER_INITIALIZE(tmc10_rgn_data);

    result = aec_trigger.Load(entry, alignment);
    if (result)
    {
        result = tmc10_rgn_data.Load(entry, alignment);
    }

    return result;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Load chromatix_tmc10_coreType
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool chromatix_tmc10_coreType::Load(
    ParameterFileSymbolTableEntry* const entry,
    BYTE const alignment)
{
    bool result = true;

    PARAMETER_INITIALIZE(mod_tmc10_aec_data);


    // Reading Array mod_tmc10_aec_data from symbol table
    // Do not check result value here because the count value needs to be initialized in this block
    {
        mod_tmc10_aec_dataCount = 0;
        SYMBOL_TABLE_ENTRY_READ(entry, result, mod_tmc10_aec_dataCount, alignment)
        constexpr const CHAR* const file {__FILE__};
        constexpr INT32 line {LINE_CONST};
        ParameterFileSymbolTableEntry  pointerEntryCls;
        ParameterFileSymbolTableEntry* const pointerEntry = &pointerEntryCls;
        if (result)
        {
            result = entry->Table->ReadPointerEntry(pointerEntryCls, entry, alignment);
        }

        if (result)
        {
            mod_tmc10_aec_dataID = pointerEntry->ID;
            DupObjInfo* mod_tmc10_aec_dataDupInfo = pointerEntry->GetDupObject(mod_tmc10_aec_dataCount, file, line);
            if (mod_tmc10_aec_dataDupInfo != nullptr && mod_tmc10_aec_dataDupInfo->Object != nullptr)
            {
                mod_tmc10_aec_data = (mod_tmc10_aec_dataType*)mod_tmc10_aec_dataDupInfo->Object;
                if (mod_tmc10_aec_data == nullptr)
                {
                    if (mod_tmc10_aec_dataDupInfo->File != nullptr)
                    {
                        PARAMETER_LOG_ERROR(entry->SetManager, "mod_tmc10_aec_data duplicate found but object is nullptr: %s:%d, RefCount: %d\n",
                            mod_tmc10_aec_dataDupInfo->File, mod_tmc10_aec_dataDupInfo->Line, mod_tmc10_aec_dataDupInfo->RefCount);
                    }
                    else
                    {
                        PARAMETER_LOG_ERROR(entry->SetManager, "mod_tmc10_aec_data duplicate found but Object is nullptr: <nullptr file ptr>:%d, RefCount: %d\n",
                            mod_tmc10_aec_dataDupInfo->Line, mod_tmc10_aec_dataDupInfo->RefCount);
                    }
                }
            }

            if (mod_tmc10_aec_data == nullptr)
            {
                if (result && mod_tmc10_aec_dataCount > 0)
                {
                    mod_tmc10_aec_data = PARAMETER_NEW mod_tmc10_aec_dataType[mod_tmc10_aec_dataCount];
                    (void)pointerEntry->SetDupObject(mod_tmc10_aec_data, mod_tmc10_aec_dataCount, file, line);
                }
                if (!result || mod_tmc10_aec_data == nullptr)
                {
                    if (mod_tmc10_aec_dataCount > 0)
                    {
                        PARAMETER_LOG_ERROR(entry->SetManager, "Failed to allocate mod_tmc10_aec_data\n");
                    }
                    mod_tmc10_aec_dataCount = 0;
                }
                for (UINT32 i = 0; result && i < mod_tmc10_aec_dataCount; i++)
                {
                    result = mod_tmc10_aec_data[i].Load(pointerEntry, alignment);
                }
            }
        }
        if (mod_tmc10_aec_data == nullptr)
        {
            mod_tmc10_aec_dataCount = 0;
        }
    }

    return result;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Load chromatix_tmc10_reserveType
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool chromatix_tmc10_reserveType::Load(
    ParameterFileSymbolTableEntry* const entry,
    BYTE const alignment)
{
    bool result = true;

    PARAMETER_INITIALIZE(use_gtm);
    PARAMETER_INITIALIZE(use_ltm);
    PARAMETER_INITIALIZE(use_la);
    PARAMETER_INITIALIZE(use_gamma);
    PARAMETER_INITIALIZE(reserve_rsv_para1);
    PARAMETER_INITIALIZE(reserve_rsv_para2);
    PARAMETER_INITIALIZE(reserve_rsv_para3);
    PARAMETER_INITIALIZE(reserve_rsv_para4);
    PARAMETER_INITIALIZE(reserve_rsv_para5);

    SYMBOL_TABLE_ENTRY_READ(entry, result, use_gtm, alignment)
    SYMBOL_TABLE_ENTRY_READ(entry, result, use_ltm, alignment)
    SYMBOL_TABLE_ENTRY_READ(entry, result, use_la, alignment)
    SYMBOL_TABLE_ENTRY_READ(entry, result, use_gamma, alignment)
    SYMBOL_TABLE_ENTRY_READ(entry, result, reserve_rsv_para1, alignment)
    SYMBOL_TABLE_ENTRY_READ(entry, result, reserve_rsv_para2, alignment)
    SYMBOL_TABLE_ENTRY_READ(entry, result, reserve_rsv_para3, alignment)
    SYMBOL_TABLE_ENTRY_READ(entry, result, reserve_rsv_para4, alignment)
    SYMBOL_TABLE_ENTRY_READ(entry, result, reserve_rsv_para5, alignment)

    return result;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Load chromatix_tmc10Type
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool chromatix_tmc10Type::Load(
    ParameterFileSymbolTableEntry* const entry,
    BYTE const alignment)
{
    bool result = true;

    PARAMETER_INITIALIZE(enable_section);
    PARAMETER_INITIALIZE(module_version);
    PARAMETER_INITIALIZE(parameter_version);
    PARAMETER_INITIALIZE(control_variables);
    PARAMETER_INITIALIZE(control_method);
    PARAMETER_INITIALIZE(chromatix_tmc10_reserve);
    PARAMETER_INITIALIZE(chromatix_tmc10_core);

    result = enable_section.Load(entry, alignment);
    if (result)
    {
        result = module_version.Load(entry, alignment);
    }
    if (result)
    {
        result = parameter_version.Load(entry, alignment);
    }
    if (result)
    {
        result = control_variables.Load(entry, alignment);
    }
    if (result)
    {
        result = control_method.Load(entry, alignment);
    }
    if (result)
    {
        result = chromatix_tmc10_reserve.Load(entry, alignment);
    }
    if (result)
    {
        result = chromatix_tmc10_core.Load(entry, alignment);
    }

    return result;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Load enable_sectionStruct
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool chromatix_tmc10Type::enable_sectionStruct::Load(
    ParameterFileSymbolTableEntry* const entry,
    BYTE const alignment)
{
    bool result = true;

    PARAMETER_INITIALIZE(adrc_isp_enable);

    SYMBOL_TABLE_ENTRY_READ(entry, result, adrc_isp_enable, alignment)

    return result;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Load module_versionStruct
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool chromatix_tmc10Type::module_versionStruct::Load(
    ParameterFileSymbolTableEntry* const entry,
    BYTE const alignment)
{
    bool result = true;

    PARAMETER_INITIALIZE(major_revision);
    PARAMETER_INITIALIZE(minor_revision);
    PARAMETER_INITIALIZE(incr_revision);

    SYMBOL_TABLE_ENTRY_READ(entry, result, major_revision, alignment)
    SYMBOL_TABLE_ENTRY_READ(entry, result, minor_revision, alignment)
    SYMBOL_TABLE_ENTRY_READ(entry, result, incr_revision, alignment)

    return result;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Load parameter_versionStruct
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool chromatix_tmc10Type::parameter_versionStruct::Load(
    ParameterFileSymbolTableEntry* const entry,
    BYTE const alignment)
{
    bool result = true;

    PARAMETER_INITIALIZE(revision);


    // Reading String revision from symbol table
    {
        UINT32 count = 0;
        SYMBOL_TABLE_ENTRY_READ(entry, result, count, alignment)
        ParameterFileSymbolTableEntry stringPointerEntryCls;
        ParameterFileSymbolTableEntry* const stringPointerEntry = &stringPointerEntryCls;
        if (result)
        {
            result = entry->Table->ReadPointerEntry(stringPointerEntryCls, entry, alignment);
        }
        SYMBOL_TABLE_ENTRY_READ_STRING(stringPointerEntry, result, revision, count, alignment)
    }

    return result;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Load control_variablesStruct
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool chromatix_tmc10Type::control_variablesStruct::Load(
    ParameterFileSymbolTableEntry* const entry,
    BYTE const alignment)
{
    bool result = true;

    PARAMETER_INITIALIZE(control_var_type);

    result = chromatix_tmc10Type::control_variablesStruct::ReadEnum_control_var_typeEnum(entry, control_var_type, alignment);

    return result;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Load control_methodStruct
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool chromatix_tmc10Type::control_methodStruct::Load(
    ParameterFileSymbolTableEntry* const entry,
    BYTE const alignment)
{
    bool result = true;

    PARAMETER_INITIALIZE(aec_exp_control);

    result = ispglobalelements::ReadEnum_tuning_control_aec_type(entry, aec_exp_control, alignment);

    return result;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Unload chromatix_tmc10_coreType
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void chromatix_tmc10_coreType::Unload(ParameterFileSymbolTable* const __table)
{
    if (__table != nullptr && __table->ReleaseDupObject(mod_tmc10_aec_data))
    {
        PARAMETER_DELETE[] mod_tmc10_aec_data;
    }
    mod_tmc10_aec_data = nullptr;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Unload chromatix_tmc10Type
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void chromatix_tmc10Type::Unload(ParameterFileSymbolTable* const __table)
{
    parameter_version.Unload(__table);
    chromatix_tmc10_core.Unload(__table);
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Unload parameter_versionStruct
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void chromatix_tmc10Type::parameter_versionStruct::Unload(ParameterFileSymbolTable* const __table)
{
    if (__table != nullptr && __table->ReleaseDupObject(revision))
    {
        PARAMETER_DELETE[] revision;
    }
    revision = nullptr;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Release the ParameterModule object
/// Requires that this object was created via a ParameterSetManager::GetModule call
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void chromatix_tmc10Type::Release()
{
    ((chromatix_tmc10TypeClass*)this)->Release();
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Returns the ParameterModule::Modified value
/// The Modified value is incremented when the module's data or pointer values change
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
UINT32 chromatix_tmc10Type::GetModified()
{
    return ((chromatix_tmc10TypeClass*)this)->Modified;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Create module, and return a new object
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
ParameterModule* chromatix_tmc10TypeClass::Create(
    ModuleData*    moduleData,
    DefaultModule* const defModule) const
{
    chromatix_tmc10TypeClass* cls = nullptr;

    if (PARAMETER_STRCMP(Type, moduleData->Type) == 0 && Version.Value == moduleData->Version.Value)
    {
        const UINT32 id = moduleData->DuplicateID > 0 ? moduleData->DuplicateID : moduleData->ID;

        ParameterModule* const module = defModule->Find(id);
        if (module != nullptr)
        {
            cls = (chromatix_tmc10TypeClass*)module;
        }
        else
        {
            cls = PARAMETER_NEW chromatix_tmc10TypeClass(GetName(moduleData->Type), moduleData);
            cls->SymbolTableID = id;
            defModule->AddModule((ParameterModule*)cls);
        }
    }
    else if (Version.Value != moduleData->Version.Value)
    {
        PARAMETER_LOG_ERROR(moduleData->SetManager, "Unable to create chromatix_tmc10TypeClass, binary version (%lld) does not match source version (%lld)\n",
            moduleData->Version.Value, Version.Value);
    }
    else
    {
        PARAMETER_LOG_ERROR(moduleData->SetManager, "Unable to create chromatix_tmc10TypeClass, binary name (%s) does not match source name (%s)\n",
            moduleData->Type, Type);
    }

    return (ParameterModule*)cls;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Load module data, return true is successful
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool chromatix_tmc10TypeClass::LoadModule(
    ParameterFileSymbolTable* const table)
{
    bool result = true;

    if (!Loaded)
    {
        BYTE const alignment = table->Alignment;
        chromatix_tmc10TypeClass* const cls = this;

        ParameterFileSymbolTableEntry  entryObj;
        ParameterFileSymbolTableEntry* const entry = &entryObj;
        // With EzTune enabled, the unused upper byte of SymbolTableID value can change, so clear it
        result = table->Find(entryObj, SymbolTableID & 0x00ffffff);

        if (!result)
        {
            PARAMETER_LOG_ERROR(entry->SetManager, "chromatix_tmc10TypeClass::LoadModule Failed to find module ID (%d) in Symbol Table\n", SymbolTableID);
        }

        if (result)
        {
            result = cls->Load(entry, alignment);
        }
        Modified++;
        Loaded = true;
    }

    if (!result)
    {
        /// Module failed to fully load, unload it
        UnloadModule();
    }

    return result;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// Unload module data
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void chromatix_tmc10TypeClass::UnloadModule()
{
    if (Loaded)
    {
        if (Data != nullptr && Data->SymbolTable != nullptr)
        {
            Unload(Data->SymbolTable);
        }
        else
        {
            PARAMETER_LOG_ERROR(Data != nullptr ? Data->SetManager : nullptr, "UnloadModule %s called with nullptr "
                "Data->SymbolTable object\n", Name);
            Unload(nullptr);
        }
        Loaded = false;
    }
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/// /// Release the ParameterModule object
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void chromatix_tmc10TypeClass::Release()
{
    ParameterModule::Release();
}

TMC_1_0_0_NAMESPACE_END
